@@ -28,10 +28,12 @@
 #include "qtractorPluginListView.h"
 
 #include "qtractorInsertPlugin.h"
+#include "qtractorMidiControlPlugin.h"
 
 #include "qtractorObserverWidget.h"
 
 #include "qtractorMidiControlObserverForm.h"
+#include "qtractorMidiControlPluginWidget.h"
 
 #include "qtractorSpinBox.h"
 
@@ -103,6 +105,8 @@ qtractorPluginForm::qtractorPluginForm (
 	m_iDirtyCount = 0;
 	m_iUpdate     = 0;
 
+	m_pMidiControlPluginWidget = nullptr;
+
 	m_pDirectAccessParamMenu = new QMenu();
 	m_ui.DirectAccessParamPushButton->setMenu(m_pDirectAccessParamMenu);
 
@@ -153,6 +157,9 @@ qtractorPluginForm::qtractorPluginForm (
 	QObject::connect(m_ui.ReturnsToolButton,
 		SIGNAL(clicked()),
 		SLOT(returnsSlot()));
+	QObject::connect(m_ui.AutoConnectCheckBox,
+		SIGNAL(toggled(bool)),
+		SLOT(autoConnectSlot(bool)));
 	QObject::connect(m_ui.AuxSendBusNameComboBox,
 		SIGNAL(activated(int)),
 		SLOT(changeAuxSendBusNameSlot(int)));
@@ -298,6 +305,21 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 	int iRow = 0;
 	int iColumn = 0;
 
+	qtractorMidiControlPlugin *pMidiControlPlugin = nullptr;
+	if (typeHint == qtractorPluginType::Control && pType->index() == 0)
+		pMidiControlPlugin = static_cast<qtractorMidiControlPlugin *> (m_pPlugin);
+	if (pMidiControlPlugin) {
+		m_pMidiControlPluginWidget = new qtractorMidiControlPluginWidget(this);
+		m_pMidiControlPluginWidget->setMidiControlPlugin(pMidiControlPlugin);
+		QObject::connect(m_pMidiControlPluginWidget,
+			SIGNAL(bipolarChanged()),
+			SLOT(updateParamRangeSlot()));
+		pGridLayout->addWidget(m_pMidiControlPluginWidget, iRow, iColumn);
+		m_ui.AutoConnectCheckBox->setChecked(
+			pMidiControlPlugin->isControlAutoConnect());
+		++iRow;
+	}
+
 	iColumnsPerPage += (iColumnsPerPage - 1); // Plus gap columns!
 	if (!m_paramWidgets.isEmpty())
 		pGridLayout->setColumnStretch(iColumn, 1);
@@ -335,7 +357,8 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 
 	// Show insert tool options...
 	const bool bInsertPlugin = (typeHint == qtractorPluginType::Insert);
-	if (bInsertPlugin) {
+	const bool bMidiControlPlugin = (pMidiControlPlugin != nullptr);
+	if (bInsertPlugin || bMidiControlPlugin) {
 		if (pType->index() > 0) { // index == channels > 0 => Audio insert.
 			m_ui.SendsToolButton->setIcon(QIcon::fromTheme("itemAudioPortOut"));
 			m_ui.ReturnsToolButton->setIcon(QIcon::fromTheme("itemAudioPortIn"));
@@ -344,8 +367,9 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 			m_ui.ReturnsToolButton->setIcon(QIcon::fromTheme("itemMidiPortIn"));
 		}
 	}
-	m_ui.SendsToolButton->setVisible(bInsertPlugin);
+	m_ui.SendsToolButton->setVisible(bInsertPlugin || bMidiControlPlugin);
 	m_ui.ReturnsToolButton->setVisible(bInsertPlugin);
+	m_ui.AutoConnectCheckBox->setVisible(bMidiControlPlugin);
 
 	// Show aux-send tool options...
 	const bool bAuxSendPlugin = (typeHint == qtractorPluginType::AuxSend);
@@ -460,6 +484,38 @@ void qtractorPluginForm::updateDirtyCount (void)
 }
 
 
+// Update specific MIDI Controller send auto-connect state.
+void qtractorPluginForm::updateMidiControlAutoConnect (void)
+{
+	if (m_pPlugin == nullptr)
+		return;
+
+	qtractorPluginType *pType = m_pPlugin->type();
+	if (pType == nullptr)
+		return;
+
+	bool bAutoConnect = false;;
+
+	if (pType->typeHint() == qtractorPluginType::Control
+		&& pType->index() == 0) {
+		qtractorMidiControlPlugin *pMidiControlPlugin
+			= static_cast<qtractorMidiControlPlugin *> (m_pPlugin);
+		if (pMidiControlPlugin) {
+			const bool bOldAutoConnect
+				= m_ui.AutoConnectCheckBox->isChecked();
+			bAutoConnect = pMidiControlPlugin->isControlAutoConnect();
+			if (m_pMidiControlPluginWidget &&
+				( bAutoConnect && !bOldAutoConnect) ||
+				(!bAutoConnect &&  bOldAutoConnect)) {
+				m_pMidiControlPluginWidget->dirtyNotify();
+			}
+		}
+	}
+
+	m_ui.AutoConnectCheckBox->setChecked(bAutoConnect);
+}
+
+
 // Update specific aux-send bus name settings.
 void qtractorPluginForm::updateAuxSendBusName (void)
 {
@@ -946,6 +1002,20 @@ void qtractorPluginForm::returnsSlot (void)
 }
 
 
+// Auto-connect (MIDI Controller Send) slot.
+void qtractorPluginForm::autoConnectSlot ( bool bOn )
+{
+	if (m_pMidiControlPluginWidget) {
+		qtractorMidiControlPlugin *pMidiControlPlugin
+			= m_pMidiControlPluginWidget->midiControlPlugin();
+		if (pMidiControlPlugin) {
+			pMidiControlPlugin->setControlAutoConnect(bOn);
+			m_pMidiControlPluginWidget->dirtyNotify();
+		}
+	}
+}
+
+
 // Audio bus name (aux-send) select slot.
 void qtractorPluginForm::changeAuxSendBusNameSlot ( int iAuxSendBusName )
 {
@@ -1074,6 +1144,23 @@ void qtractorPluginForm::changeDirectAccessParamSlot (void)
 }
 
 
+void qtractorPluginForm::updateParamRangeSlot (void)
+{
+	if (m_pPlugin == nullptr)
+		return;
+
+	if (m_pMidiControlPluginWidget == nullptr)
+		return;
+
+	if (m_iUpdate > 0)
+		return;
+
+	QListIterator<qtractorPluginParamWidget *> iter(m_paramWidgets);
+	while (iter.hasNext())
+		iter.next()->updateParamRange();
+}
+
+
 // Parameter-widget refreshner-loader.
 void qtractorPluginForm::refresh (void)
 {
@@ -1176,6 +1263,11 @@ void qtractorPluginForm::clear (void)
 	m_paramWidgets.clear();
 
 	m_pDirectAccessParamMenu->clear();
+
+	if (m_pMidiControlPluginWidget) {
+		delete m_pMidiControlPluginWidget;
+		m_pMidiControlPluginWidget = nullptr;
+	}
 }
 
 
@@ -1596,6 +1688,27 @@ void qtractorPluginParamWidget::refresh (void)
 }
 
 
+// Special range updater.
+void qtractorPluginParamWidget::updateParamRange (void)
+{
+	const float fValue = m_pParam->value();
+
+	if (m_pSpinBox) {
+		const bool bSpinBox = m_pSpinBox->blockSignals(true);
+		m_pSpinBox->setMinimum(m_pParam->minValue());
+		m_pSpinBox->setMaximum(m_pParam->maxValue());
+		m_pSpinBox->setValue(fValue);
+		m_pSpinBox->blockSignals(bSpinBox);
+	}
+
+	if (m_pSlider) {
+		const bool bSlider = m_pSlider->blockSignals(true);
+		m_pSlider->setValue(m_pSlider->scaleFromValue(fValue));
+		m_pSlider->blockSignals(bSlider);
+	}
+}
+
+
 // Parameter automation curve status update/refresh.
 void qtractorPluginParamWidget::updateCurveButton (void)
 {
